Package: GenomicDataCommons
Type: Package
Title: NIH / NCI Genomic Data Commons Access
Description: Programmatically access the NIH / NCI Genomic Data Commons
        RESTful service.
Version: 1.35.1
Date: 2025-05-12
Authors@R: c( person("Martin", "Morgan",
        email="martin.morgan@roswellpark.org", role=c("aut")),
        person("Sean", "Davis", email="seandavi@gmail.com",
        role=c("aut", "cre")),
        person("Marcel", "Ramos",
        email = "marcel.ramos@sph.cuny.edu", role = "ctb"))
License: Artistic-2.0
Depends: R (>= 4.1.0)
Imports: stats, httr, xml2, jsonlite, utils, rlang, readr,
        GenomicRanges, IRanges, dplyr, rappdirs, tibble, tidyr
Suggests: BiocStyle, knitr, rmarkdown, DT, testthat, listviewer,
        ggplot2, GenomicAlignments, Rsamtools, BiocParallel,
        TxDb.Hsapiens.UCSC.hg38.knownGene, VariantAnnotation, maftools,
        R.utils, data.table
biocViews: DataImport, Sequencing
URL: https://bioconductor.org/packages/GenomicDataCommons,
        http://github.com/Bioconductor/GenomicDataCommons,
        http://bioconductor.github.io/GenomicDataCommons/
BugReports: https://github.com/Bioconductor/GenomicDataCommons/issues/new
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/GenomicDataCommons
git_branch: devel
git_last_commit: 06edbb7
git_last_commit_date: 2025-11-07
Repository: Bioconductor 3.23
Date/Publication: 2025-11-07
NeedsCompilation: no
Packaged: 2025-11-07 22:44:25 UTC; biocbuild
Author: Martin Morgan [aut],
  Sean Davis [aut, cre],
  Marcel Ramos [ctb]
Maintainer: Sean Davis <seandavi@gmail.com>
