# Generated by roxygen2: do not edit by hand

S3method(aggregations,GDCQuery)
S3method(aggregations,GDCResponse)
S3method(available_expand,GDCQuery)
S3method(available_expand,character)
S3method(available_fields,GDCQuery)
S3method(available_fields,character)
S3method(count,GDCQuery)
S3method(count,GDCResponse)
S3method(default_fields,GDCQuery)
S3method(default_fields,character)
S3method(entity_name,GDCQuery)
S3method(entity_name,GDCResults)
S3method(expand,GDCQuery)
S3method(facet,GDCQuery)
S3method(field_description,GDCQuery)
S3method(field_description,character)
S3method(filter,GDCQuery)
S3method(get_facets,GDCQuery)
S3method(get_filter,GDCQuery)
S3method(id_field,GDCQuery)
S3method(id_field,GDCResults)
S3method(ids,GDCManifest)
S3method(ids,GDCQuery)
S3method(ids,GDCResponse)
S3method(ids,GDCResults)
S3method(manifest,GDCcasesResponse)
S3method(manifest,GDCfilesResponse)
S3method(manifest,gdc_files)
S3method(print,gdc_endpoints)
S3method(print,gdc_parameters)
S3method(response,GDCQuery)
S3method(results,GDCQuery)
S3method(results,GDCResponse)
S3method(results_all,GDCQuery)
S3method(results_all,GDCResponse)
S3method(select,GDCQuery)
export(aggregations)
export(annotations)
export(available_expand)
export(available_fields)
export(available_values)
export(cases)
export(cnv_occurrences)
export(cnvs)
export(count)
export(default_fields)
export(endpoints)
export(entity_name)
export(expand)
export(facet)
export(field_description)
export(files)
export(filter)
export(gdc_cache)
export(gdc_client)
export(gdc_client_version_validate)
export(gdc_clinical)
export(gdc_set_cache)
export(gdc_token)
export(gdcdata)
export(genes)
export(get_facets)
export(get_filter)
export(grep_fields)
export(id_field)
export(ids)
export(make_filter)
export(manifest)
export(mapping)
export(parameters)
export(projects)
export(query)
export(readDNAcopy)
export(readHTSeqFile)
export(response)
export(response_all)
export(results)
export(results_all)
export(select)
export(slicing)
export(ssm_occurrences)
export(ssms)
export(status)
export(transfer)
export(transfer_help)
export(write_manifest)
import(GenomicRanges)
importFrom(IRanges,IRanges)
importFrom(dplyr,bind_rows)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(httr,headers)
importFrom(httr,progress)
importFrom(httr,stop_for_status)
importFrom(httr,write_disk)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,unbox)
importFrom(rappdirs,app_dir)
importFrom(readr,read_tsv)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,f_env)
importFrom(rlang,f_rhs)
importFrom(rlang,is_formula)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(utils,menu)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_text)
