#' GenomicDataCommons: A package for interfacing with the NCI GDC
#' 
#' @section finding data:
#' 
#' \itemize{
#' \item{\code{\link{query}}}
#' \item{\code{\link{cases}}}
#' \item{\code{\link{projects}}}
#' \item{\code{\link{files}}}
#' \item{\code{\link{annotations}}}
#' \item{\code{\link{mapping}}}
#' }
#'
#' @section downloading data:
#' data
#'
"_PACKAGE"
